from collections import defaultdict
from lxml import etree
from StringIO import StringIO

import oauth2
import textwrap

from outcome_response import OutcomeResponse
from utils import InvalidLTIConfigError

REPLACE_REQUEST = 'replaceResult'
DELETE_REQUEST = 'deleteResult'
READ_REQUEST = 'readResult'

class OutcomeRequest():
    '''
    Class for consuming & generating LTI Outcome Requests.

    Outcome Request documentation: http://www.imsglobal.org/lti/v1p1pd/ltiIMGv1p1pd.html#_Toc309649691

    This class can be used both by Tool Providers and Tool Consumers, though
    they each use it differently. The TP will use it to POST an OAuth-signed
    request to the TC. A TC will use it to parse such a request from a TP.
    '''
    def __init__(self, opts = defaultdict(lambda: None)):
        # Store specified options in our options member
        self.options = defaultdict(lambda: None)
        for (key, val) in opts.iteritems():
            self.options[key] = val

    @staticmethod
    def from_post_request(post_request):
        '''
        Convenience method for creating a new OutcomeRequest from a request
        object.

        post_request is assumed to be a Django HttpRequest object
        '''
        request = OutcomeRequest()
        request.options['post_request'] = post_request
        request.process_xml(post_request.POST)
        return request

    def post_replace_result(self, score):
        '''
        POSTs the given score to the Tool Consumer with a replaceResult.
        '''
        self.options['operation'] = REPLACE_REQUEST
        self.options['score'] = score
        self.post_outcome_request()

    def post_delete_result(self):
        '''
        POSTs a deleteRequest to the Tool Consumer.
        '''
        self.options['operation'] = DELETE_REQUEST
        self.post_outcome_request()

    def post_read_result(self):
        '''
        POSTS a readResult to the Tool Consumer.
        '''
        self.options['operation'] = READ_REQUEST
        self.post_outcome_request()

    def is_replace_request(self):
        '''
        Check whether this request is a replaceResult request.
        '''
        return self.options['operation'] == REPLACE_REQUEST

    def is_delete_request(self):
        '''
        Check whether this request is a deleteResult request.
        '''
        return self.options['operation'] == DELETE_REQUEST

    def is_read_request(self):
        '''
        Check whether this request is a readResult request.
        '''
        return self.options['operation'] == READ_REQUEST

    def was_outcome_post_successful(self):
        return self.options['outcome_response'] and\
                self.options['outcome_response'].is_success()

    def post_outcome_request(self):
        '''
        POST an OAuth signed request to the Tool Consumer.
        '''
        if not self.has_required_attributes():
            raise InvalidLTIConfigError(textwrap.dedent('OutcomeRequest does not have all required attributes'))

        consumer = oauth2.Consumer(key = self.options['consumer_key'],
                secret =  self.options['consumer_secret'])

        client = oauth2.Client(consumer)

        response, content = client.request(
                self.options['lis_outcome_service_url'],
                'POST',
                body = self.generate_request_xml(),
                headers = { 'Content-Type': 'application/xml' })

        self.options['outcome_response'] =\
                OutcomeResponse.from_post_response(response)

    def process_xml(self, xml):
        '''
        Parse Outcome Request data from XML.
        '''
        context = etree.iterparse(StringIO(xml))
        for action, elem in context:
            if 'imsx_messageIdentifier' in elem.tag:
                self.options['message_identifier'] = elem.text
            elif 'imsx_sourcedId' in elem.tag:
                self.options['lis_result_sourcedid'] = elem.text
            elif 'deleteResultRequest' in elem.tag:
                self.options['operation'] = DELETE_REQUEST
            elif 'readResultRequest' in elem.tag:
                self.options['operation'] = READ_REQUEST
            elif 'replaceResult' in elem.tag:
                self.options['operation'] = REPLACE_REQUEST
            elif 'textString' in elem.tag:
                self.options['score'] = elem.text
            elif 'sourcedId' in elem.tag:
                self.options['lis_result_sourcedid'] = elem.text

    def has_required_attributes(self):
        return self.options['consumer_key'] != None\
                and self.options['consumer_secret'] != None\
                and self.options['lis_outcome_service_url'] != None\
                and self.options['lis_result_sourcedid'] != None\
                and self.options['operation'] != None

    def generate_request_xml(self):
        root = etree.Element('imsx_POXEnvelopeResponse', xmlns =
                'http://www.imsglobal.org/lis/oms1p0/pox')

        header = etree.SubElement(root, 'imsx_POXHeader')
        header_info = etree.SubElement(header, 'imsx_POXResponseHeaderInfo')
        version = etree.SubElement(header_info, 'imsx_version')
        version.text = 'V1.0'
        message_identifier = etree.SubElement(header_info,
                'imsx_messageIdentifier')
        message_identifier.text = self.options['message_identifier']
        body = etree.SubElement(root, 'imsx_POXBody')
        request = etree.SubElement(body, '%s%s' %(self.options['operation'],
            'Request'))
        record = etree.SubElement(request, 'resultRecord')
        guid = etree.SubElement(record, 'sourceGUID')
        guid.text = self.options['lis_result_sourcedid']
        
        if self.options['score']:
            result = etree.SubElement(record, 'result')
            result_score = etree.SubElement(result, 'resultScore')
            language = etree.SubElement(result_score, 'language')
            language.text = 'en'
            text_string = etree.SubElement(result_score, 'textString')
            text_string.text = self.options['score'].__str__()

        return etree.tostring(root, xml_declaration = True, encoding = 'utf-8')



import sys, os
cwd = os.getcwd()
sys.path.append(cwd)

from tool_provider import ToolProvider
from tool_consumer import ToolConsumer

def create_params_tp():
    '''
    Creates a set of launch parameters for a ToolConsumer.
    '''
    return {
          "lti_message_type": "basic-lti-launch-request",
          "lti_version": "LTI-1p0",
          "resource_link_id": "c28ddcf1b2b13c52757aed1fe9b2eb0a4e2710a3",
          "lis_result_sourcedid": "261-154-728-17-784",
          "lis_outcome_service_url": "http://localhost/lis_grade_passback",
          "launch_presentation_return_url": "http://example.com/lti_return",
          "custom_param1": "custom1",
          "custom_param2": "custom2",
          "ext_lti_message_type": "extension-lti-launch",
          "roles": "Learner,Instructor,Observer"
    }

def create_test_tp():
    '''
    Returns a new ToolProvider.
    '''
    return ToolProvider('hi', 'oi', create_params_tp())

def create_params_tc():
    '''
    Creates a set of launch parameters for a ToolConsumer.
    '''
    params = create_params_tp()
    params.update({
            #'resource_link_id': '120988f929-274612',
            'user_id': '292832126',
            'roles': 'Instructor',
            'lis_person_name_full': 'Jane Q. Public',
            'lis_person_contact_email_primary': 'user@school.edu',
            'context_id': '456434513',
            'context_title': 'Design of Personal Environments',
            'context_label': 'SI182',
            'lti_version': 'LTI-1p0',
            'lti_message_type': 'basic-lti-launch-request',
            'tool_consumer_instance_guid': 'lmsng.school.edu',
            'tool_consumer_instance_description': 'University of School (LMSng)',
    })
    return params

def create_test_tc(params = None):
    '''
    Returns a new ToolConsumer.
    '''
    params = create_params_tc() if params == None else params
    tc = ToolConsumer('12345', 'secret', params)
    tc.launch_url = 'http://dr-chuck.com/ims/php-simple/tool.php'
    tc.timestamp = '1251600739'
    tc.nonce = 'c8350c0e47782d16d2fa48b2090c1d8f'
    tc.set_non_spec_param('lis_person_sourced_id', 'school.edu:user')
    tc.set_non_spec_param('basiclti_submit', 'Launch Endpoint with BasicLTI Data')

    return tc



from uuid import uuid1

def generate_identifier():
    return uuid1().__str__()

class InvalidLTIConfigError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return repr(self.value)



from collections import defaultdict
from lxml import etree

CODE_MAJOR_CODES = [
        'success',
        'processing',
        'failure',
        'unsupported'
]

SEVERITY_CODES = [
        'status',
        'warning',
        'error'
]

class OutcomeResponse():
    '''
    This class consumes & generates LTI Outcome Responses.
  
    Response documentation: http://www.imsglobal.org/lti/v1p1pd/ltiIMGv1p1pd.html#_Toc309649691

    Error code documentation: http://www.imsglobal.org/gws/gwsv1p0/imsgws_baseProfv1p0.html#1639667

    This class can be used by both Tool Providers and Tool Consumers, though
    each will use it differently. TPs will use it to partse the result of an
    OutcomeRequest to the TC. A TC will use it to generate proper response XML
    to send back to a TP.
    ''' 
    def __init__(self, opts = defaultdict(lambda: None)):
        self.code_major = None

        # Store specified options in our options member
        for (key, val) in opts.iteritems():
            self.options[key] = val

    def from_post_response(self, post_response):
        '''
        Convenience method for creating a new OutcomeResponse from a response
        object.
        '''
        response = OutcomeResponse()
        response.post_response = post_response
        response.reponse_code = post_response.code
        xml = post_response.body
        response.process_xml(xml)
        return response

    def is_success(self):
        return self.code_major == 'success'

    def is_processing(self):
        return self.code_major == 'processing'

    def is_failure(self):
        return self.code_major == 'failure'

    def is_unsupported(self):
        return self.code_major == 'unsupported'

    def has_warning(self):
        return self.severity == 'warning'

    def has_error(self):
        return self.severity == 'error'

    def process_xml(xml):
        '''
        Parse OutcomeResponse data form XML.
        '''
        # TODO
        pass

    def generate_response_xml(self):
        '''
        Generate XML based on the current configuration.
        '''
        root = etree.Element('imsx_POXEnvelopeResponse', xmlns =
                'http://www.imsglobal.org/lis/oms1p0/pox',
                xml_declaration = True, encoding = 'utf-8')

        header = etree.SubElement(root, 'imsx_POXHeader')
        header_info = etree.SubElement(header, 'imsx_POXResponseHeaderInfo')
        version = etree.SubElement(header_info, 'imsx_version')
        version.text = 'V1.0'
        message_identifier = etree.SubElement(header_info,
                'imsx_messageIdentifier')
        message_identifier.text = self.options['message_identifier']
        status_info = etree.SubElement(header, 'imsx_statusInfo')
        code_major = etree.SubElement(status_info, 'imsx_codeMajor')
        code_major.text = self.options['code_major']
        severity = etree.SubElement(status_info, 'imsx_severity')
        severity.text = self.options['severity']
        description = etree.SubElement(status_info, 'imsx_description')
        description.text = self.options['description']
        message_ref_identifier = etree.SubElement(status_info,
                'imsx_messageRefIdentifier')
        message_ref_identifier.text = self.options['message_ref_identifier']
        operation_ref_identifier= etree.SubElement(status_info,
                'imsx_operationRefIdentifier')
        operation_ref_identifier.message_ref_identifiertexa = self.options['operation']

        body = etree.SubElement(root, 'imsx_POXBody')
        request = etree.SubElement(body, '%s%s' %(self.options['operation'],
            'Request'))
        record = etree.SubElement(request, 'resultRecord')
        guid = etree.SubElement(record, 'sourceGUID')
        guid.text = self.options['lis_result_sourcedid']
        
        if self.options['score']:
            result = etree.SubElement(record, 'result')
            result_score = etree.SubElement(result, 'resultScore')
            language = etree.SubElement(result_score, 'language')
            language.text = 'en'
            text_string = etree.SubElement(result_score, 'textString')
            text_string.text = self.options['score']



from collections import defaultdict

# List of the standard launch parameters for an LTI launch
LAUNCH_DATA_PARAMETERS = [
        'context_id',
        'context_label',
        'context_title',
        'context_type',
        'launch_presentation_css_url',
        'launch_presentation_document_target',
        'launch_presentation_height',
        'launch_presentation_locale',
        'launch_presentation_return_url',
        'launch_presentation_width',
        'lis_course_offering_sourcedid',
        'lis_course_section_sourcedid',
        'lis_outcome_service_url',
        'lis_person_contact_email_primary',
        'lis_person_name_family',
        'lis_person_name_full',
        'lis_person_name_given',
        'lis_person_sourcedid',
        'lis_result_sourcedid',
        'lti_message_type',
        'lti_version',
        'oauth_callback',
        'oauth_consumer_key',
        'oauth_nonce',
        'oauth_signature',
        'oauth_signature_method',
        'oauth_timestamp',
        'oauth_version',
        'resource_link_description',
        'resource_link_id',
        'resource_link_title',
        'roles',
        'tool_consumer_info_product_family_code',
        'tool_consumer_info_version',
        'tool_consumer_instance_contact_email',
        'tool_consumer_instance_description',
        'tool_consumer_instance_guid',
        'tool_consumer_instance_name',
        'tool_consumer_instance_url',
        'user_id',
        'user_image'
]

class LaunchParamsMixin():
    def __init__(self):
        # These dictionaries return a 'None' object when accessing a key that
        # is not in the dictionary.
        self.launch_params = defaultdict(lambda: None)
        self.custom_params = defaultdict(lambda: None)
        self.ext_params = defaultdict(lambda: None)

    def roles(self, roles_list):
        '''
        Set the roles for the current launch.

        Full list of roles can be found here:
        http://www.imsglobal.org/LTI/v1p1pd/ltiIMGv1p1pd.html#_Toc309649700

        LIS roles include:
        * Student
        * Faculty
        * Member
        * Learner
        * Instructor
        * Mentor
        * Staff
        * Alumni
        * ProspectiveStudent
        * Guest
        * Other
        * Administrator
        * Observer
        * None
        '''
        if roles_list and isinstance(roles_list, list):
            self.roles = [].extend(roles_list)
        elif roles_list and isinstance(roles_list, str):
            self.roles = [role.lower() for role in roles_list.split(',')]

    def process_params(self, params):
        '''
        Populates the launch data from a dictionary. Only cares about keys in
        the LAUNCH_DATA_PARAMETERS list, or that start with 'custom_' or
        'ext_'.
        '''
        for key in params.iterkeys():
            if key in LAUNCH_DATA_PARAMETERS:
                if key == 'roles':
                    # Roles are a list of ',' delimited strings
                    self.roles = [role.lower() for role in
                            params[key].split(',')]
                else:
                    self.launch_params[key] = params[key]
            elif 'custom_' in key:
                self.custom_params[key] = params[key]
            elif 'ext_' in key:
                self.ext_params[key] = params[key]

    def set_custom_param(self, key, val):
      self.custom_params['custom_' + key] = val

    def get_custom_param(self, key):
      return self.custom_params['custom_' + key]

    def set_non_spec_param(self, key, val):
      self.non_spec_params[key] = val

    def get_non_spec_param(self, key):
      return self.non_spec_params[key]

    def set_ext_param(self, key, val):
      self.ext_params['ext_' + key] = val

    def get_ext_param(self, key):
      return self.ext_params['ext_' + key]
    
    def to_params(self):
        '''
        Createa a new dictionary with all launch data. Custom / Extension keys
        will be included. Roles are set as a ',' separated string.
        '''
        params = {}
        custom_params = {}
        for key in self.custom_params:
            custom_params[key] = self.custom_params[key]
        ext_params = {}
        for key in self.ext_params:
            ext_params[key] = self.ext_params[key]
        params.update(self.launch_params)
        params.update(custom_params)
        params.update(ext_params)
        return params



from tool_consumer import ToolConsumer
from test_helper import create_test_tc
import unittest

class TestToolConsumer(unittest.TestCase):
    def test_signature(self):
        '''
        Should generate correct oauth signature.
        '''
        tc = create_test_tc()
        result = tc.generate_launch_data()
        self.assertNotEqual(result, None)
        self.assertEqual(result['oauth_signature'],
                'TPFPK4u3NwmtLt0nDMP1G1zG30U=')

    def test_url_query_parameters(self):
        '''
        Should generate a correct signature with URL query parameters.
        '''
        tc = create_test_tc()
        tc.launch_url = 'http://dr-chuck.com/ims/php-simple/tool.php?a=1&b=2&c=3%20%26a'
        result = tc.generate_launch_data()
        self.assertNotEqual(result, None)
        self.assertEquals(result['oauth_signature'], 
                'uF7LooyefQN5aocx7UlYQ4tQM5k=')
        self.assertEquals(result['c'], '3 &a')

    def test_signature_port(self):
        '''
        Should generate a correct signature with a non-standard port.
        '''
        tc = ToolConsumer('12345', 'secret', {'resource_list_id': 1})
        tc.timestamp = '1251600739'
        tc.nonce = 'c8350c0e47782d16d2fa48b2090c1d8f'

        def test_url(url, sig):
            tc.launch_url = url
            ld = tc.generate_launch_data()
            self.assertNotEqual(ld, None)
            self.assertEquals(ld['oauth_signature'], sig)

        test_url('http://dr-chuck.com:123/ims/php-simple/tool.php', 'HdHJri8Z7OhnMhxX27hSPB5W+SI=')
        test_url('http://dr-chuck.com/ims/php-simple/tool.php', 'bTcODyqQSdogpn1mJAugGB2c2F4=')
        test_url('http://dr-chuck.com:80/ims/php-simple/tool.php', 'bTcODyqQSdogpn1mJAugGB2c2F4=')
        test_url('http://dr-chuck.com:443/ims/php-simple/tool.php', 'n0P6aFgyv6ikNsMiNNG/KjxMZ8w=')
        test_url('https://dr-chuck.com/ims/php-simple/tool.php', '9DoVeq1RYnidTgF71Zg16SNJFpY=')
        test_url('https://dr-chuck.com:443/ims/php-simple/tool.php', '9DoVeq1RYnidTgF71Zg16SNJFpY=')
        test_url('https://dr-chuck.com:80/ims/php-simple/tool.php', '4L1f5SctEX0num3GPElvMKq2w+s=')
        test_url('https://dr-chuck.com:80/ims/php-simple/tool.php?oi=hoyt', 'dvvQchwqhDH1nFGzWbgVxmcUysc=')

    def test_uri_query_parameters(self):
        '''
        Should include URI query parameters.
        '''
        tc = ToolConsumer('12345', 'secret', {
            'resource_link_id': 1,
            'user_id': 2
            })
        tc.launch_url = 'http://www.yahoo.com?a=1&b=2'
        result = tc.generate_launch_data()
        self.assertNotEqual(result, None)
        self.assertEqual(result['a'], '1')
        self.assertEqual(result['b'], '2')

    def test_overite_uri_query_parameters(self):
        '''
        Should not allow overwriting other parameters from the URI query
        string.
        '''
        tc = ToolConsumer('12345', 'secret', {
            'resource_link_id': 1,
            'user_id': 2
            })
        tc.launch_url = 'http://www.yahoo.com?user_id=123&lti_message_type=1234'
        result = tc.generate_launch_data()
        self.assertNotEqual(result, None)
        self.assertEqual(result['user_id'], '2')
        self.assertEqual(result['lti_message_type'],
                'basic-lti-launch-request')



from test_helper import create_test_tp, create_params_tp
import unittest

class TestToolProvider(unittest.TestCase):
    def setUp(self):
        self.params = create_params_tp()
        self.tp = create_test_tp()

    def test_outcome_service(self):
        '''
        Should recognize an outcome service.
        '''
        self.assertTrue(self.tp.is_outcome_service())
        self.tp.launch_params['lis_result_sourcedid'] = None
        self.assertFalse(self.tp.is_outcome_service())

    def test_return_url_with_messages(self):
        '''
        Should generate a return url with messages.
        '''
        self.assertEqual(self.tp.build_return_url(),
                self.params['launch_presentation_return_url'])
        self.tp.launch_params['lti_errormsg'] = 'user error message'
        self.tp.launch_params['lti_errorlog'] = 'lms error log'
        self.tp.launch_params['lti_msg'] = 'user message'
        self.tp.launch_params['lti_log'] = 'lms message'
        self.assertEqual(self.tp.build_return_url(),
                self.params['launch_presentation_return_url'] +
                '?lti_errormsg=user%20error%20message&lti_errorlog=lms%20error%20log&lti_msg=user%20message&lti_log=lms%20message')

    def test_roles(self):
        '''
        Should recognize the roles.
        '''
        self.assertTrue(self.tp.is_student())
        self.assertTrue(self.tp.is_instructor())
        self.assertTrue(self.tp.has_role('Observer'))
        self.assertFalse(self.tp.has_role('administrator'))

    def test_username(self):
        '''
        Should find the best username.
        '''
        self.assertEqual(self.tp.username('guy'), 'guy')
        self.tp.launch_params['lis_person_name_full'] = 'full'
        self.assertEqual(self.tp.username('guy'), 'full')
        self.tp.launch_params['lis_person_name_family'] = 'family'
        self.assertEqual(self.tp.username('guy'), 'family')
        self.tp.launch_params['lis_person_name_given'] = 'given'
        self.assertEqual(self.tp.username('guy'), 'given')



from test_helper import create_test_tp
from outcome_request import OutcomeRequest

import unittest

class TestOutcomeRequest(unittest.TestCase):
    def setUp(self):
        self.tp = create_test_tp()
        self.expected_xml = '<?xml version="1.0" encoding="UTF-8"?><imsx_POXEnvelopeRequest xmlns="http://www.imsglobal.org/lis/oms1p0/pox"><imsx_POXHeader><imsx_POXRequestHeaderInfo><imsx_version>V1.0</imsx_version><imsx_messageIdentifier>123456789</imsx_messageIdentifier></imsx_POXRequestHeaderInfo></imsx_POXHeader><imsx_POXBody>%s</imsx_POXBody></imsx_POXEnvelopeRequest>'
        self.replace_result_xml = self.expected_xml[:] %('<replaceResultRequest><resultRecord><sourcedGUID><sourcedId>261-154-728-17-784</sourcedId></sourcedGUID><result><resultScore><language>en</language><textString>5</textString></resultScore></result></resultRecord></replaceResultRequest>')
        self.read_result_xml = self.expected_xml[:] %('<readResultRequest><resultRecord><sourcedGUID><sourcedId>261-154-728-17-784</sourcedId></sourcedGUID></resultRecord></readResultRequest>')
        self.delete_result_xml = self.expected_xml[:] %('<deleteResultRequest><resultRecord><sourcedGUID><sourcedId>261-154-728-17-784</sourcedId></sourcedGUID></resultRecord></deleteResultRequest>')
         
    #def test_post_replace_result(self):
    #    '''
    #    Should post replaceResult rquest.
    #    '''
    #    self.tp.post_replace_result(5)
    #    self.assetFalse(self.tp.last_outcome_success)

    #def test_post_read_result(self):
    #    '''
    #    Should post readResult request.
    #    '''
    #    self.tp.post_read_result()

    #def  test_post_delete_result(self):
    #    '''
    #    Should post deleteResult request.
    #    '''
    #    self.tp.post_delete_result()

    def test_parse_replace_result_xml(self):
        '''
        Should parse replaceResult XML.
        '''
        request = OutcomeRequest()
        request.process_xml(self.replace_result_xml)
        self.assertEqual(request.options['operation'], 'replaceResult')
        self.assertEqual(request.options['lis_result_sourcedid'], '261-154-728-17-784')
        self.assertEqual(request.options['message_identifier'], '123456789')
        self.assertEqual(request.options['score'], '5')

    def test_parse_read_result_xml(self):
        '''
        Should parse readResult XML.
        '''
        request = OutcomeRequest()
        request.process_xml(self.read_result_xml)
        self.assertEqual(request.options['operation'], 'readResult')
        self.assertEqual(request.options['lis_result_sourcedid'], '261-154-728-17-784')
        self.assertEqual(request.options['message_identifier'], '123456789')
        self.assertEqual(request.options['score'], None)

    def test_parse_delete_result_xml(self):
        '''
        Should parse deleteRequest XML.
        '''
        request = OutcomeRequest()
        request.process_xml(self.delete_result_xml)
        self.assertEqual(request.options['operation'], 'deleteResult')
        self.assertEqual(request.options['lis_result_sourcedid'], '261-154-728-17-784')
        self.assertEqual(request.options['message_identifier'], '123456789')
        self.assertEqual(request.options['score'], None)



from launch_params import LaunchParamsMixin
from request_validator import RequestValidatorMixin
from outcome_request import OutcomeRequest
from urllib import quote
from collections import defaultdict

class ToolProvider(LaunchParamsMixin, RequestValidatorMixin, object):
    '''
    Implements the LTI Tool Provider.
    '''
    
    def __init__(self, consumer_key, consumer_secret, params = {}):
        '''
        Create new ToolProvider.
        '''
        super(ToolProvider, self).__init__()

        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.non_spec_params = defaultdict(lambda: None)
        self.outcome_requests = []
        self.process_params(params)

    def has_role(self, role):
        '''
        Check whether the Launch Paramters set the role.
        '''
        return self.roles and role.lower() in self.roles

    def is_student(self):
        '''
        Convenience method for checking if the user has 'learner' or 'student'
        role.
        '''
        return self.has_role('learner') or self.has_role('student')

    def is_instructor(self):
        '''
        Convenience method for checking if user has 'instructor', 'faculty'
        or 'staff' role.
        '''
        return self.has_role('instructor') or self.has_role('faculty') or\
                self.has_role('staff')

    def is_launch_request(self):
        '''
        Check if the request was an LTI Launch Request.
        '''
        return self.launch_params['lti_message_type'] ==\
                'basic-lti-launch-request'

    def is_outcome_service(self):
        '''
        Check if the Tool Launch expects an Outcome Result.
        '''
        return (self.launch_params['lis_outcome_service_url']and
                self.launch_params['lis_result_sourcedid'])

    def username(self, default = None):
        '''
        Return the full, given, or family name if set.
        '''
        if self.launch_params['lis_person_name_given']:
            return self.launch_params['lis_person_name_given']
        elif self.launch_params['lis_person_name_family']:
            return self.launch_params['lis_person_name_family']
        elif self.launch_params['lis_person_name_full']:
            return self.launch_params['lis_person_name_full']
        else:
            return default

    def post_replace_result(self, score):
        '''
        POSTs the given score to the Tool Consumer with a replaceResult.

        Returns OutcomeResponse object and stores it in self.outcome_request
        '''
        return self.new_request().post_replace_result(score)

    def post_delete_result(self):
        '''
        POSTs a delete request to the Tool Consumer.
        '''
        return self.new_request().post_delete_result()

    def post_read_result(self):
        '''
        POSTs the given score to the Tool Consumer with a replaceResult, the
        returned OutcomeResponse will have the score.
        '''
        return self.new_request().post_read_result()

    def last_outcome_request(self):
        '''
        Returns the most recent OutcomeRequest.
        '''
        return self.outcome_requests[-1]

    def last_outcome_success(self):
        '''
        Convenience method for determining the success of the last
        OutcomeRequest.
        '''
        return self.last_outcome_request and\
                self.last_outcome_request.is_outcome_post_successful()

    def build_return_url(self):
        '''
        If the Tool Consumer sent a return URL, add any set messages to the
        URL.
        '''
        if not self.launch_params['launch_presentation_return_url']:
            return None

        messages = []
        for message in ['lti_errormsg', 'lti_errorlog', 'lti_msg', 'lti_log']:
            if self.launch_params[message]:
                messages.append('%s=%s' %(message,
                    quote(self.launch_params[message])))

        q_string = '?' + '&'.join(messages) if messages else ''
        return self.launch_params['launch_presentation_return_url'] + q_string

    def new_request(self):
        opts = defaultdict(lambda: None)
        opts = { 
                'consumer_key': self.consumer_key, 
                'consumer_secret': self.consumer_secret, 
                'lis_outcome_service_url': self.launch_params['lis_outcome_service_url'], 
                'lis_result_sourcedid': self.launch_params['lis_result_sourcedid']
                }
        self.outcome_requests.append(OutcomeRequest(opts = opts))
        self.last_outcome_request = self.outcome_requests[-1]
        return self.last_outcome_request



import oauth2

class RequestValidatorMixin():
    '''
    A 'mixin' for OAuth request validation.
    '''
    def __init__(self):
        self.oauth_server = oauth2.Server()
        self.oath_consumer = oauth2.Consumer(self.consumer_key,
                self.consumer_secret)

    def is_valid_request(self, request, handle_error = True):
        '''
        Validates an OAuth request using the python-oauth2 library:
            https://github.com/simplegeo/python-oauth2

        '''
        try:
            self.oauth_server.verify_request(request, self.consumer)
        except oauth2.MissingSignature, e:
            if handle_error:
                return False
            else:
                raise e

    def valid_request(self, request):
        '''
        Check whether the OAuth-signed request is valid and throw error if not.
        '''
        self.is_valid_request(request, False)

    def request_oauth_nonce(self):
        '''
        Convenience method for getting the oauth nonce from the request.
        '''
        return self.oauth_signature_validator\
                and self.oath_signature_validator.request.oath_nonce

    def request_oauth_timestamp(self):
        '''
        Convenience method for getting the oath timestamp from the request.
        '''
        return self.oath_signature_validator\
                and self.oauth_signature_validator.request.oauth_timestamp



from collections import defaultdict

# Namespaces used for parsing configuration XML
LTI_NAMESPACES = {
        "xmlns": 'http://www.imsglobal.org/xsd/imslticc_v1p0',
        "blti": 'http://www.imsglobal.org/xsd/imsbasiclti_v1p0',
        "lticm": 'http://www.imsglobal.org/xsd/imslticm_v1p0',
        "lticp": 'http://www.imsglobal.org/xsd/imslticp_v1p0',
        }

class ToolConfig():
    '''
    Object used to represent LTI configuration.

    Capable of creating and reading the Common Cartridge XML representation of
    LTI links as described here:
        http://www.imsglobal.org/LTI/v1p1pd/ltiIMGv1p1pd.html#_Toc309649689

    TODO: Usage description
    '''
    def __init__(self, opts = defaultdict(lambda: None)):
        '''
        Create a new ToolConfig with the given options.
        '''
        self.custom_params = opts.pop('custom_params') or\
                defaultdict(lambda: None)
        self.extensions = opts.pop('extensions') or\
                defaultdict(lambda: None)
        self.opts = defaultdict(lambda: None)

        for (key, val) in opts.iteritems():
            self.opts[key] = val

    def create_from_xml(self):
        '''
        Create a ToolConfig from the given XML.
        '''
        # TODO: WTF!?
        pass

    def set_custom_param(self, key, val):
        '''
        Set a custom parameter to provided value.
        '''
        self.custom_params[key] = val

    def get_custom_param(self, key):
        '''
        Gets a custom parameter. It not yet set, returns None object.
        '''
        return self.custom_params[key]

    def set_ext_params(self, ext_key, ext_params):
        '''
        Set the extension parameters for a specific vendor.
        '''
        self.extensions[ext_key] = ext_params

    def get_ext_params(self, ext_key):
        '''
        Get extension paramaters for provided extension. It not set, returns None object.
        '''
        return self.extensions[ext_key]

    def set_ext_param(self, ext_key, param_key, val):
        '''
        Set the provided parameter in a set of extension parameters.
        '''
        if not self.extensions[ext_key]:
            self.extensions[ext_key] = defaultdict(lambda: None)
        self.extensions[ext_key][param_key] = val

    def get_ext_param(self, ext_key, param_key):
        '''
        Get specific param in set of provided extension parameters.
        '''
        return self.extensions[ext_key][param_key] if self.extensions[ext_key]\
                else None

    def process_xml(xml):
        '''
        Parse tool configuration data out of the Common Cartridge LTI link XML.
        '''
        # TODO Parse XML
        pass

    def to_xml(opts = defaultdict(lambda: None)):
        '''
        Generate XML from the current settings.
        '''
        # TODO Generte XML
        pass






from outcome_response import OutcomeResponse 

import unittest
import textwrap

class TestOutcomeResponse(unittest.TestCase):
    def setUp(self):
        self.response_xml = textwrap.dedent('''\
                <?xml version="1.0" encoding="UTF-8"?>
                <imsx_POXEnvelopeResponse xmlns="http://www.imsglobal.org/lis/oms1p0/pox">
                <imsx_POXHeader>
                <imsx_POXResponseHeaderInfo>
                <imsx_version>V1.0</imsx_version>
                <imsx_messageIdentifier></imsx_messageIdentifier>
                <imsx_statusInfo>
                <imsx_codeMajor>success</imsx_codeMajor>
                <imsx_severity>status</imsx_severity>
                <imsx_description></imsx_description>
                <imsx_messageRefIdentifier>123456789</imsx_messageRefIdentifier>
                <imsx_operationRefIdentifier>replaceResult</imsx_operationRefIdentifier>
                </imsx_statusInfo>
                </imsx_POXResponseHeaderInfo>
                </imsx_POXHeader>
                <imsx_POXBody>
                <replaceResultResponse></replaceResultResponse>
                </imsx_POXBody>
                </imsx_POXEnvelopeResponse>
                ''')

    def mock_response(self, response_xml):
        pass

    def test_parse_replace_result_response_xml(self):
        '''
        Should parse replaceResult response XML.
        '''
        result = OutcomeResponse()
        self.assertTrue(result.is_success())
        self.assertEqual(result.code_major, 'success')
        self.assertEqual(result.severity, 'status')
        self.assertEqual(result.description, None)
        self.assertEqual(result.message_ref_identifier, '123456789')
        self.assertEqual(result.operation, 'replaceResult')
        self.assertEqual(result.score, None)

    def test_parse_read_result_response_xml(self):
        '''
        Should parse readResult response XML.
        '''
        pass

    def test_parse_delete_result_response_xml(self):
        '''
        Should parse deleteResult response XML.
        '''
        result = OutcomeResponse()
        self.assertTrue(result.is_success())
        self.assertEqual(result.code_major, 'success')
        self.assertEqual(result.severity, 'status')
        self.assertEqual(result.description, None)
        self.assertEqual(result.message_ref_identifier, '123456789')
        self.assertEqual(result.operation, 'deleteResult')
        self.assertEqual(result.score, None)

    def test_recognize_failure_response(self):
        '''
        Should recognize a failure response.
        '''
        result = OutcomeResponse()
        self.assertTrue(result.is_failure())

    def test_generate_response_xml(self):
        '''
        Should generate response XML.
        '''
        result = OutcomeResponse()
        # TODO



from setuptools import setup, find_packages

setup(
    name = 'ims_lti_py',
    version = '0.3',
    description = 'A Python library to help implement IMS LTI tool consumers and providers',
    author = 'Anson MacKeracher',
    author_email = 'anson@tophatmonocle.com',
    url = 'https://github.com/tophatmonocle/ims_lti_py',
    packages = find_packages(),
    install_requires = ['lxml', 'oauth2'],
    license = 'MIT License',
    keywords = 'lti',
    zip_safe = True,
    test_suite = 'tests',
)



from test_helper import create_test_tp, create_test_tc, create_params_tp

import unittest
import httplib2

class DontTestLaunchParams():
    def test_process_params(self):
        '''
        Should process parameters.
        '''
        for (key, val) in self.params.iteritems():
            if not 'custom_' in key\
                    and not 'ext_' in key\
                    and not 'roles' in key:
                self.assertEquals(self.tool.launch_params[key], val)

        # Test roles
        self.assertTrue(sorted(self.tool.roles) == sorted(['learner',\
                'instructor', 'observer']))

    def test_custom_extension_parameters(self):
        '''
        Should handle custom/extension parameters.
        '''
        self.assertEquals(self.tool.get_custom_param('param1'), 'custom1')
        self.assertEquals(self.tool.get_custom_param('param2'), 'custom2')
        self.assertEquals(self.tool.get_ext_param('lti_message_type'),
                'extension-lti-launch')
        self.tool.set_custom_param('param3', 'custom3')
        self.tool.set_ext_param('user_id', 'bob')

        params = self.tool.to_params()
        self.assertEquals(params['custom_param1'], 'custom1')
        self.assertEquals(params['custom_param2'], 'custom2')
        self.assertEquals(params['custom_param3'], 'custom3')
        self.assertEquals(params['ext_lti_message_type'],
                'extension-lti-launch')
        self.assertEquals(params['ext_user_id'], 'bob')

    def test_invalid_request(self):
        '''
        Should not accept invalid request.
        '''
        # Create request, 
        request = httplib2.Http.request('/test?key=value', method = 'POST')

        # Validate request
        self.too.is_valid_request(request)

class TestProviderLaunchParams(unittest.TestCase, DontTestLaunchParams):
    '''
    Tests the LaunchParamsMixin component of the ToolProvider.
    '''
    def setUp(self):
        self.params = create_params_tp()
        self.tool = create_test_tp()

class TestConsumerLaunchParams(unittest.TestCase, DontTestLaunchParams):
    '''
    Tests the LaunchParamsMixin component of the ToolConsumer.
    '''
    def setUp(self):
        self.params = create_params_tp()
        self.tool = create_test_tc(params = self.params)



from collections import defaultdict
from urlparse import urlparse

import textwrap
import oauth2

from launch_params import LaunchParamsMixin
from request_validator import RequestValidatorMixin
from utils import InvalidLTIConfigError

class ToolConsumer(LaunchParamsMixin, RequestValidatorMixin, object):
    def __init__(self, consumer_key, consumer_secret, params =
            defaultdict(lambda: None)):
        '''
        Create new ToolConsumer.
        '''
        super(ToolConsumer, self).__init__()

        # Prevent key errors
        params_dict = defaultdict(lambda: None)
        params_dict.update(params)

        self.consumer_key = consumer_key
        self.consumer_secret = consumer_secret
        self.non_spec_params = defaultdict(lambda: None)

        self.launch_url = params_dict['launch_url']
        self.process_params(params_dict)

    def set_config(self, config):
        '''
        Set launch data from a ToolConfig.
        '''
        # TODO: Implement ToolConfig
        pass

    def has_required_params(self):
        '''
        Check if required parameters for a tool launch are set.
        '''
        return self.consumer_key and\
                self.consumer_secret and\
                self.launch_params['resource_link_id'] and\
                self.launch_url

    def generate_launch_data(self):
        # Validate params
        if not self.has_required_params():
            raise InvalidLTIConfigError(textwrap.dedent('ToolConsumer does not have all required attributes'))

        params = self.to_params()
        params['lti_version'] = 'LTI-1.0'
        params['lti_message_type'] = 'basic-lti-launch-request'

        # Parse URL to get host
        uri = urlparse(self.launch_url)

        if uri.port == None:
            host = uri.host
        else:
            host = uri.host + ':' + uri.port

        # Get new OAuth consumer
        consumer = oauth2.Consumer(key = self.options['consumer_key'],\
                secret = self.options['consumer_secret'])
        
        path = uri.path
        path = '/' if path == '' else path

        options = {
                'oauth_nonce': self.nonce,
                'oauth_timestamp': self.timestamp,
                'oauth_scheme': 'body'
                }
        request = oauth2.Request(method = 'POST', 
                url = self.launch_url,
                parameters = options)

        signature_method = oauth2.SignatureMethod_HMAC_SHA1()
        request.sign_request(signature_method, consumer, None)

        # Request was made by an HTML form in the user's browser. Revert the
        # escapage and return the hash of post parameters ready for embedding
        # in an html view.
        dic = {}
        for param in request.body.split('&'):
            for key, val in param.split('='):
                dic[key] = val
        return dic
